import logging
from datetime import datetime, date, timedelta
import json

from requests import get, codes
from requests.exceptions import MissingSchema, InvalidSchema, ConnectionError, InvalidURL

log = logging.getLogger('dynamat2050')

# 'url': 'https://subdomain.dynamat2050.com/ProductService.svc',
# 'username': '',
# 'password': ''

class DynamatError(Exception): pass

class Client():

    def __init__(self, url=None, username=None, password=None, **kwargs):
        self.url = url
        self.headers = {"Accept": "application/json"}
        self.fmt = "%Y-%m-%d %H:%M:%S"
        self.auth = (username, password)

    def test(self):
        response = self.get('meterlist') # potentially raises exceptions
        return {'status': 'OK', 'message': 'Successfully connected'}

    def meters(self):
        response = self.get('meterlist') # potentially raises exceptions
        return [{
            'identifier': m['MeterId'],
            'label': m['Description'].strip()
        } for m in response.json()]

    def meter_data(self, meter_id, from_, to_):
        response = self.consumption(meter_id, from_, to_, "Minute", 30)
        try:
            data = response.json()
        except:
            print(response)
        if data:
            return json.dumps(data).encode('utf-8')

    def consumption(self, meter_id, from_, to, interval, n_intervals):
        """Make an actual request for energy data to the server at %url"""
        payload = {
            "ID": meter_id,
            "from": from_.strftime(self.fmt),
            "to": to.strftime(self.fmt),
            "interval": interval,
            "intervalQty": n_intervals
        }
        return self.get('Consumption', params=payload)

    def get(self, endpoint, headers=None, **kwargs):
        """make requests relative to the base_url
        merge default headers
        include any etag passed as an If-Match header"""
        headers = headers or {}
        headers.update(self.headers)
        url = "{}/{}".format(self.url, endpoint)
        try:
            response = get(url, headers=headers, auth=self.auth, verify=False, **kwargs) # verify is set to false for self-signed https
        # except SSLError:
        #     raise InvalidCertificate()
        except ConnectionError as exc:
            try:
                connect_to_google()
            except ConnectionError as e:
                raise SourceConnectionError(exc)
            raise DynamatError("Could not connect - check url")
        except (MissingSchema, InvalidSchema):
            raise DynamatError("Missing schema in url (e.g. https://)")
        except InvalidURL:
            raise DynamatError("Invalid url: '{}'".format(self.url))

        if response.status_code == codes.unauthorized:
            raise DynamatError("Invalid credentials")
        if response.status_code == codes.bad_request:
            raise DynamatError("Bad request")
        if response.status_code == codes.not_found:
            raise DynamatError("Invalid URL - not a dynamat2050 server")
        if response.status_code == codes.forbidden:
            raise DynamatError("Unauthorised")
        if response.status_code == codes.server_error:
            raise DynamatError("Problem with Dynamat server")
        return response



import logging
import sys
import os
import json
from collections import deque
from datetime import datetime, timedelta

from dynamat2050 import Client

log = logging.getLogger("download")
logging.basicConfig(level=logging.INFO)

now = datetime.now()
one_year = timedelta(days=365)

class ArgParser(object):
    """
    Check args for command line script
    First arg is the config file
    Second arg is the command
    """
    def __init__(self, args):
        print("\n==========================")
        print(" downloading dynamat data ")
        print("==========================\n")
        self.args = deque(args)
        self.script = self.args.popleft()
        try:
            self.config_uri = self.args.popleft()
        except IndexError:
            log.error("Must provide a config file as first argument")
            exit(1)
        if not os.path.exists(self.config_uri):
            log.error("First argument must be a valid config file")
            exit(1)
        try:
            self.command = self.args.popleft()
        except IndexError:
            log.error("No command provided")
            exit(1)
        try:
            self.outdir = self.args.popleft()
        except IndexError:
            log.error("No output directory provided")
            exit(1)
        self.args = list(self.args)

def client_from_config(config):
    url, username, password = config.pop('url'), config.pop('username'), config.pop('password')
    return Client(url=url, username=username, password=password)

def main(argv=sys.argv):
    """
    Command line script
    Args are parsed and given to the edinet thing.
    """
    args = ArgParser(argv)
    with open(args.config_uri) as f:
        config = json.load(f)
    client = client_from_config(config)
    if args.command == "meters":
        log.info("attempting to get meters list")
        outfile = os.path.join(args.outdir, "meters.json")
        with open(outfile, 'w') as f:
            json.dump(client.meters(), f, indent=2, sort_keys=True)
    elif args.command == "meter_data":
        for meter in config['meters']:
            outfile = os.path.join(args.outdir, "meter_{}.json".format(meter))
            with open(outfile, 'w') as f:
                data = client.meter_data(meter, now - one_year, now).decode('utf-8')
                json.dump(data, f, indent=2, sort_keys=True)
    else:
        print("Command '{}' not recognised, use one of [meters, meter_data]".format(args.command))

if __name__ == "__main__":
    main()
